'3DBUTN.BAS - Using two character generator tables simultaneously,
'             a Windows-style 3-D command button is displayed in the
'             center of the screen. When a key is pressed, the button
'             is "depressed." When another key is pressed, the button
'             is "released." Press <Esc> to terminate this program.

' To run this program in the QB 4.X environment, type the following at
' the DOS prompt:
'
'                   QB 3DBUTN /L QB.QLB
'
' To run this program in the QBX environment, type the following at
' the DOS prompt:
'
'                   QBX 3DBUTN /L QBX.QLB
'
' This program requires the "InterruptX" routine and the QB(x).BI file
' included with QuickBASIC/BASIC PDS 7.x. The code from LISTING1.BAS
' must also be "loaded" (Alt-F-L) into the environment or placed in a
' QuickLibrary before this program can be executed.

DEFINT A-Z                          'Good practice!

REM $INCLUDE: 'QBX.BI'               'Use QBX.BI if using PDS

'Declarations for the functions we will be using from LISTING1.BAS

DECLARE SUB DefineChar (BitMask$, Ascii%, TableNum%)
DECLARE SUB LoadDefaultFont ()
DECLARE FUNCTION MonitorType% ()
DECLARE SUB TwoFontsOff ()
DECLARE SUB TwoFontsOn ()

DIM Reg AS RegTypeX                     'For use with InterruptX

SELECT CASE MonitorType%                'What type of display?
	CASE -1                             'VGA
		CharHeight% = 16                'An 8X16 character set
		CharSet% = 6                    'Use BIOS function 6 to get
										' a pointer to this CGT.
	CASE -2                             'EGA
		CharHeight% = 14                'An 8X14 character set
		CharSet% = 2                    'Use BIOS function 2 to get
										' a pointer to this CGT.
	CASE ELSE                           'Neither an EGA or VGA
		PRINT "An EGA/VGA color adapter is not present!"
		END
END SELECT

CLS
PRINT "Defining fonts...."          'So we know what's going on

Reg.ax = &H1130                     'Get font information (Ega/Vga)
Reg.bx = CharSet% * 256             'Pointer to current CGT into BH
CALL InterruptX(&H10, Reg, Reg)     'Call BIOS Int 10h

' In order to get the 3-D effect when a button is depressed, we're
' going to load each character from the default CGT0, shift the
' bits down and to the right one pixel, and store this "new"
' character in CGT1.

DEF SEG = Reg.es
Offset& = Reg.bp                        'Offset to the first char
FOR Ascii% = 0 TO 255                   'For each ASCII character
	FOR Byte% = 1 TO CharHeight% - 2    'and for each scan line (-1)
		BitMap% = PEEK(Offset&) \ 2     'Get scan line and shift 1 bit
		Character$ = Character$ + CHR$(BitMap%) 'Store it
		Offset& = Offset& + 1           'Point to next scan line
	NEXT                                'Repeat for each scan line
	Offset& = Offset& + 2               'Compensate for skipping last
	Character$ = CHR$(0) + CHR$(0) + Character$ ' scan line.
	CALL DefineChar(Character$, Ascii%, 1)  'Put in CGT1
	Character$ = ""
NEXT
DEF SEG

' Let's redefine 8 key line-drawing characters to enhance the 3-D
' effect. First, the ones in CGT0 which will be displayed while the
' button is in the "released" state. The bit maps are in the DATA
' statements at the end of this program.

FOR Table% = 0 TO 1
	FOR Z% = 1 TO 8
		READ Ascii%                        'Read the Ascii code
		FOR Byte% = 1 TO CharHeight%       'Then the bit map for each
			READ BitMap%                   ' scan line.
			'If using an EGA, skip the center two scan lines so we end
			'up with an 8 X 16 matrix instead of an 8 X 16.
			IF CharHeight% = 16 OR ((Byte% < 8 OR Byte% > 9) AND CharHeight% = 14) THEN
				Character$ = Character$ + CHR$(BitMap%)
			END IF
		NEXT
		CALL DefineChar(Character$, Ascii%, Table%)
		Character$ = ""
	NEXT
NEXT

COLOR 0, 7                          'Set color of black on white
CLS                                 'Clear the screen
CALL TwoFontsOn                     'Enable both CGTs
Fg% = 0                             'Use a foreground color of black

DO                                  'Display the button
	COLOR Fg%                       'Change the foreground color
	LOCATE 10, 28: PRINT CHR$(218); STRING$(20, 196); CHR$(191)
	LOCATE 11, 28: PRINT CHR$(179); " 3-D Command Button "; CHR$(180)
	LOCATE 12, 28: PRINT CHR$(192); STRING$(20, 197); CHR$(217)
	A$ = INPUT$(1)                  'Wait for a key
	IF A$ = CHR$(27) THEN EXIT DO   'If Escape, exit the loop
	Fg% = Fg% XOR 8                 'Otherwise, toggle HI
LOOP                                'And display in the new state

CALL TwoFontsOff
CALL LoadDefaultFont
END

'"Released" bit maps - Placed in CGT0

DATA 179,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192
DATA 180,22,42,22,42,22,42,22,42,22,42,22,42,22,42,22,42
DATA 191,254,254,22,42,22,42,22,42,22,42,22,42,22,42,22,42
DATA 192,192,192,192,192,192,192,192,192,192,192,192,234,213,234,213,255
DATA 196,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DATA 197,0,0,0,0,0,0,0,0,0,0,0,170,85,170,85,255
DATA 217,22,42,22,42,22,42,22,42,22,42,22,170,86,170,86,254
DATA 218,255,255,192,192,192,192,192,192,192,192,192,192,192,192,192,192

'"Depressed" bit maps - Placed in CGT1

DATA 179,168,208,168,208,168,208,168,208,168,208,168,208,168,208,168,208
DATA 180,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6
DATA 191,254,86,170,86,170,6,6,6,6,6,6,6,6,6,6,6
DATA 192,168,208,168,208,168,208,168,208,168,208,168,208,168,208,168,255
DATA 196,255,85,170,85,170,0,0,0,0,0,0,0,0,0,0,0
DATA 197,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255
DATA 217,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,254
DATA 218,255,213,170,213,170,208,168,208,168,208,168,208,168,208,168,208

