
SUB BreakUpLine(ThisLine$, Indent$, Word$(), Comment$, NumWords%, OldLine$)
'-----------------------------------------------------------------------------
'Changes any comment marked by "REM" into one marked by "'"
'Breaks up the line into:
'    Indent$ = the blanks or tabs at the beginning of the line
'    Comment$ = the comment; anything after a "'" or "REM"
'    Word$() = an array of the elements in the line
'
'Creates OldLine$, which is the original line preceded by "'QB:"
'    NumWords% = the number of words found
'
'-----------------------------------------------------------------------------

'save spaces or tabs at beginning of line to preserve formatting.
   Ctr% = 1
   Indent$ =""
   Comment$ =""
   DO
      SpaceChar$ = mid$(ThisLine$, Ctr%, 1)
      IF SpaceChar$ =" " or SpaceChar$ = chr$(9) THEN
         Indent$ = Indent$ + SpaceChar$
         incr Ctr%
      ELSE
         EXIT LOOP
      END IF
   LOOP
   ThisLine$ = rtrim$(ltrim$(ThisLine$, any chr$(9) +" "))
   OldLine$ ="'QB: " + ThisLine$

'I don't like comments marked by REM, so change it to "'".
'If the string "REM" is in this line, it still may not be a real comment
'because the string may be part of a quoted string.

   IF istrue instr(ucase$(ThisLine$), "REM ") THEN
      Ctr% = 1
      InQuote% = %False
      WHILE Ctr% <= len(ThisLine$)
         TestChar$ = mid$(ThisLine$, Ctr%, 1)
         SELECT CASE TestChar$
         CASE "'"
'there's a real comment marker, so nevermind
            IF isfalse InQuote% THEN EXIT LOOP
         CASE chr$(34)
'if it's a quote mark, toggle flag between 0 and -1
            bit toggle InQuote%, 15
'or it might be the beginning of a "REM"
         CASE "R", "r"
            IF isfalse InQuote% THEN
'special case:
'Normally check for a space before and after, to make sure it's not just
'part of some other string.  If it's the first character, we can't check for
'Ctr% - 1, so make a special case to check for "REM ".
               IF Ctr% > 1 THEN
                  IF ucase$(mid$(ThisLine$, Ctr% - 1, 5)) =" REM " THEN
                     mid$(ThisLine$, Ctr%, 4) ="   '"
                     EXIT LOOP
                  END IF
               ELSE
                  IF ucase$(mid$(ThisLine$, Ctr%, 4)) ="REM " THEN
                     mid$(ThisLine$, Ctr%, 4) ="   '"
                     ThisLine$ = ltrim$(ThisLine$)
                     EXIT LOOP
                  END IF           'end of is it "REM"?
               END IF              'end of if Ctr% > 1
            END IF                 'end of isfalse InQuote%
         END SELECT                'end of select case TestChar$
         incr Ctr%
      WEND                         'end of while Ctr% <= len(ThisLine$)
   END IF                          'end of check for "'" or "REM"

'now break up the line into pieces:
   Symbol$ ="*()_+-=;:'<>,/\"
   NumWords% = 1
   CharCtr% = 0
'clear array contents
   ArraySize% = ubound(Word$())
   REDIM Word$(ArraySize%)
'if a blank line, just clear Word$() and return
   IF rtrim$(ltrim$(ThisLine$)) ="" THEN
      NumWords% = 0
      EXIT SUB
   END IF

   DO
      incr CharCtr%
      TestChar$ = mid$(ThisLine$, CharCtr%, 1)
      IF istrue InQuote% THEN
'if it's in a quoted string, disregard what it is, just append it.  If there
'is a space after the ending quote, it will be added under "case else"
         Word$(NumWords%) = Word$(NumWords%) + TestChar$
         IF TestChar$ = chr$(34) THEN Inquote% = 0
      ELSE
         SELECT CASE TestChar$
         CASE "'"
            IF isfalse Inquote% THEN
'everything else is a comment
               Comment$ = mid$(ThisLine$, CharCtr%)
               ThisLine$ = rtrim$(left$(ThisLine$, CharCtr% - 1))
               EXIT LOOP
            END IF
         CASE ";", ",", "(", ")", "-", "+", "*", "/", "\", "_", ":"
            IF Word$(NumWords%) <> "" THEN incr NumWords%
            Word$(NumWords%) = TestChar$
            incr NumWords%
         CASE "<", ">", "="
'Special case: it may be the beginning of <>, <=, =<, >=, or =>
            IF Word$(NumWords%) <> "" THEN incr NumWords%
            Word$(NumWords%) = TestChar$
            TestChar$ = mid$(ThisLine$, CharCtr% +1, 1)
            SELECT CASE TestChar$
            CASE "<", ">", "="
               Word$(NumWords%) = Word$(NumWords%) + TestChar$
               incr CharCtr%
            END SELECT
'end of this word
            incr NumWords%
         CASE chr$(34)
'it's a beginning quote; ending quotes are taken care of above
            Word$(NumWords%) = TestChar$
            Inquote% = - 1
         CASE " "
'do not include blank if it is only separating two symbols
            IF (instr(Symbol$, mid$(ThisLine$, CharCtr% - 1, 1)) <> 0) and_
                     (instr(Symbol$, mid$(ThisLine$, CharCtr% + 1, 1)) <> 0) THEN
'do nothing
            ELSE
               Word$(NumWords%) = Word$(NumWords%) + TestChar$
            END IF
'if it's the end of a string, start a new word.  Otherwise, keep adding to
'this one.
            IF ltrim$(Word$(NumWords%), any chr$(9) + " ") <> "" THEN incr NumWords%
         CASE ELSE
'it's a string character; add it
            Word$(NumWords%) = Word$(NumWords%) + TestChar$
         END SELECT
      END IF
   LOOP UNTIL CharCtr% >= len(ThisLine$)
'if the last character in the line is one of the symbols, Word$(NumWords%)
'will be "".  If there is a string of blank spaces before a comment, make
'them a part of the comment.  This will take care of either situation:
   IF rtrim$(Word$(NumWords%), any chr$(9) +" ") ="" THEN
      Comment$ = Word$(NumWords%) + Comment$
      array delete Word$(NumWords%)
      decr NumWords%
   END IF

END SUB

SUB UnMultiply (ThisLine$, NextLine$)

'save spaces or tabs at beginning of line to preserve formatting.
   Ctr% = 1
   Indent$ =""
   DO
      SpaceChar$ = mid$(ThisLine$, Ctr%, 1)
      IF SpaceChar$ =" " or SpaceChar$ = chr$(9) THEN
         Indent$ = Indent$ + SpaceChar$
         incr Ctr%
      ELSE
         EXIT LOOP
      END IF
   LOOP

'if there is a ":" it may be a multiple command line.  Make sure before
'breaking it into several lines.
   IF instr(ThisLine$, ":") > 0 THEN
'If at the end of the line, it is a label, not a multiple-command line
      IF instr(ThisLine$, ":") < len(ThisLine$) THEN
'if it gets to here the ":" is not at the end of the line.  It may be
'inside a quoted string though, so check the line one character at a time,
'toggling a flag to tell whether it is inside a pair of quotes
         CharCtr% = 1
         InQuote% = 0
         WHILE CharCtr% <= len(ThisLine$)
            TestChar$ = mid$(ThisLine$, CharCtr%, 1)
            SELECT CASE TestChar$
            CASE chr$(34)
'toggle InQuote% between 0 and -1
               bit toggle InQuote%, 15
            CASE "'"
'found a comment, no point in continuing
               IF isfalse InQuote% THEN EXIT LOOP
            case "r", "R"
'it could be a "REM" comment also:
               if ucase$(mid$(ThisLine$, CharCtr%, 4)) = "REM " then
               if isfalse InQuote% then exit loop
               end if
            CASE ":"
               IF isfalse InQuote% THEN
'tack on Indent$ so the next line will have the same indentation
                  NextLine$ = ltrim$(mid$(ThisLine$, CharCtr% + 1))
                  IF NextLine$ <> "" THEN NextLine$ = Indent$ + NextLine$
                  ThisLine$ = rtrim$(left$(ThisLine$, CharCtr% - 1))
                  EXIT LOOP
               END IF
            END SELECT
            incr CharCtr%
         WEND
      END IF           'end of is it a label
   END IF              'end of is there a ":"

END SUB


SUB RebuildLine (ThisLine$, Indent$, Word$(), Comment$, NumWords%)
'-----------------------------------------------------------------------------
'This procedure rebuilds the line from the Word$ array and returns the new
'line as ThisLine$.  It could use some improvement.
'
'-----------------------------------------------------------------------------

   ThisLine$ =""
   FOR Ctr% = 1 to NumWords%
      SELECT CASE left$(Word$(Ctr%), 1)
      CASE "-"
'"-" needs special treatment because it can be arithmetic operator or a
'negative sign in front of a number, or a separator.  This still needs some
'work.
'add spaces before but not after, except in certain cases:
         Word$(Ctr%) = rtrim$(Word$(Ctr%))
         IF right$(Word$(Ctr% - 1), 1) <> " " THEN
            Word$(Ctr%) =" " + Word$(Ctr%)
         END IF
         IF ucase$(rtrim$(ltrim$(Word$(Ctr% -2)))) ="DEFINT" THEN
'I want "DEFINT (A-Z)" instead of "DEFINT (A -Z)"
            Word$(Ctr%) = rtrim$(ltrim$(Word$(Ctr%)))
            Word$(Ctr% -1) = rtrim$(Word$(Ctr% -1))
            Word$(Ctr% +1) = ltrim$(Word$(Ctr% +1))
         END IF
         IF Word$(Ctr% -1) ="(" THEN
'I want "(-1)" instead of "( -1)"
            Word$(Ctr%) = ltrim$(Word$(Ctr%))
         END IF
      CASE "(", ")", "_"
'do nothing
      CASE ":", "<", ">", "+", "=", "*", "/", "\"
'add space before and after:
         Word$(Ctr%) = rtrim$(ltrim$(Word$(Ctr%)))
         IF right$(Word$(Ctr% -1), 1) <> " " THEN
            Word$(Ctr%) =" " + Word$(Ctr%)
         END IF
         IF left$(Word$(Ctr% +1), 1) <> " " THEN
            Word$(Ctr%) = Word$(Ctr%) +" "
         END IF
      CASE ";", ","
'add space after, unless there is already one
         IF left$(Word$(Ctr% + 1), 1) <> " " THEN
            Word$(Ctr%) = Word$(Ctr%) +" "
         END IF
'never use a space before it
         Word$(Ctr% - 1) = rtrim$(Word$(Ctr% - 1))
      CASE ELSE
'it's a string; leave it alone.
      END SELECT
   NEXT Ctr%
   FOR Ctr% = 1 to NumWords%
      ThisLine$ = ThisLine$ + Word$(Ctr%)
   NEXT Ctr%
'if line is blank, this will get rid of Indent$:
   ThisLine$ = rtrim$(Indent$ + ltrim$(ThisLine$) + Comment$)
END SUB
