'3DBUTN.BAS - Using two character generator tables simultaneously,
'             a Windows-style #-D command button is displayed in the
'             center of the screen. When a key is pressed, the button
'             "depresses." When another key is pressed, the button is
'             "released." Press <Esc> to terminate this program.

' This program requires a VGA color display, but can be easily modified
' to run on an EGA color display.

' To run this program in the QB 4.X environment, type the following at
' the DOS prompt:
'
'                   QB 3DBUTN /L QB.QLB
'
' To run this program in the QBX environment, type the following at the
' DOS prompt:
'
'                   QBX 3DBUTN /L QBX.QLB
'
' This program requires the "InterruptX" routine and the QB(x).BI file
' included with QuickBASIC/BASIC PDS 7.x. The code from LISTING1.BAS
' must also be "loaded" (Alt-F-L) into the environment or placed in a
' QuickLibrary.

DEFINT A-Z                          'Good practice!

REM $INCLUDE: 'QBX.BI'              'Use QBX.BI if using PDS

'Declarations for the functions we will be using from LISTING1.BAS

DECLARE SUB DefineChar (BitMask$, Ascii%, TableNum%)
DECLARE FUNCTION MonitorType% ()
DECLARE SUB TwoFontsOff ()
DECLARE SUB TwoFontsOn ()

DIM Reg AS RegTypeX                     'For use with InterruptX

CLS                                     'Clear the screen

SELECT CASE MonitorType%                'What type of display?
	CASE -1                             'VGA
		CharHeight% = 16                'An 8X16 character set
		CharSet% = 6                    'Use BIOS function 6 to get
										' a pointer to this CGT.
	CASE -2                             'EGA
		CharHeight% = 14                'An 8X14 character set
		CharSet% = 2                    'Use BIOS function 2 to get
										' a pointer to this CGT.
	CASE ELSE                           'Neither an EGA or VGA
		PRINT "An EGA/VGA color adapter is not present!"
		END
END SELECT

PRINT "Defining fonts...."          'So we know what's going on

Reg.AX = &H1130                     'Get font information (Ega/Vga)
'Reg.BX = CharSet%                   'Pointer to current CGT
Reg.BX = &H100                         'Pointer to current CGT
CALL InterruptX(&H10, Reg, Reg)     'Call BIOS Int 10h

PRINT "       Address: "; HEX$(Reg.ES); ":"; HEX$(Reg.BP)
PRINT "Bytes per char:"; Reg.CX  ' CharHeight%
PRINT "Rows on screen:"; Reg.DX MOD 256 + 1

' In order to get the 3-D effect when a button is depressed, we're
' going to load each character from the default CGT0, shift the
' bits down and to the right one pixel, and store this "new"
' character in CGT1.

DEF SEG = Reg.ES                        'Point default segment to CGT0
Offset& = Reg.BP                        'Offset to the first char

FOR Ascii% = 0 TO 255                   'For each ASCII character
	Character$ = ""
	PRINT "Offset: "; Offset&
	FOR Byte% = 1 TO Reg.CX             'and for each scan line (-1)
		BitMap% = PEEK(Offset&)         'Get scan line and shift 1 bit
		Character$ = Character$ + CHR$(BitMap%) 'Store it
		Offset& = Offset& + 1           'Point to next scan line
	NEXT                                'Repeat for each scan line
	GOSUB DisplayChar
	A$ = INPUT$(1)
NEXT
DEF SEG

' Let's redefine 8 key line-drawing characters to enhance the 3-D
' effect. First, the ones in CGT0 which will be displayed while the
' button is in the "released" state. The bit maps are in the DATA
' statements at the end of this program.

FOR Table% = 0 TO 1
	FOR Z% = 1 TO 8
		READ Ascii%                        'Read the Ascii code
		FOR Byte% = 1 TO CharHeight%       'Then the bit map for each
			READ BitMap%                   ' scan line.
			'If using an EGA, skip the center two scan lines so we end
			'up with an 8 X 16 matrix instead of an 8 X 16.
			IF CharHeight% = 16 OR ((Byte% < 8 OR Byte% > 9) AND CharHeight% = 14) THEN
				Character$ = Character$ + CHR$(BitMap%)
			END IF
		NEXT
		CALL DefineChar(Character$, Ascii%, Table%)
		Character$ = ""
	NEXT
NEXT

Fg% = 7
COLOR Fg%, 1
CLS
CALL TwoFontsOn
DO
	PRINT TAB(20); CHR$(218); STRING$(20, 196); CHR$(191)
'    PRINT TAB(20); CHR$(179); STRING$(20, 32); CHR$(180)
	PRINT TAB(20); CHR$(179); " 3-D Command Button "; CHR$(180)
'    PRINT TAB(20); CHR$(179); STRING$(20, 32); CHR$(180)
	PRINT TAB(20); CHR$(192); STRING$(20, 197); CHR$(217)
	A$ = INPUT$(1)
	IF A$ = CHR$(27) THEN EXIT DO
	Fg% = Fg% XOR 8
	COLOR Fg%
	LOCATE 1
LOOP

END
DisplayChar:
	PRINT "Character"; Ascii%; CHR$(Ascii%)
	PRINT "Bit", ; "7 6 5 4 3 2 1 0"
	FOR Scan% = 1 TO Reg.CX
		PRINT Scan% - 1, ;
		BitMap% = ASC(MID$(Character$, Scan%, 1))
		GOSUB PrintLine
	NEXT
RETURN

PrintLine:
	FOR Bit% = 7 TO 0 STEP -1
		IF BitMap% AND 2 ^ Bit% THEN
			PRINT " ";
		ELSE
			PRINT "  ";
		END IF
	NEXT
	PRINT
RETURN

'Pressed Goes in table 1

DATA 179,160,208,160,208,160,208,160,208,160,208,160,208,160,208,160,208
DATA 180,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3
DATA 191,0,255,171,87,171,3,3,3,3,3,3,3,3,3,3,3
DATA 192,160,208,160,208,160,208,160,208,160,208,160,208,160,208,160,255
DATA 196,0,255,170,85,170,0,0,0,0,0,0,0,0,0,0,0
DATA 197,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255
DATA 217,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,255
DATA 218,0,255,170,213,170,208,160,208,160,208,160,208,160,208,160,208


'Define our new box border characters - Un-pressed - Goes in table 0

DATA 179,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192
DATA 180,5,11,5,11,5,11,5,11,5,11,5,11,5,11,5,11
DATA 191,0,255,255,11,5,11,5,11,5,11,5,11,5,11,5,11
DATA 192,192,192,192,192,192,192,192,192,192,192,192,192,213,234,213,255
DATA 196,0,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0
DATA 197,0,0,0,0,0,0,0,0,0,0,0,0,85,170,85,255
DATA 217,5,11,5,11,5,11,5,11,5,11,5,11,85,171,85,255
DATA 218,0,255,255,192,192,192,192,192,192,192,192,192,192,192,192,192

