DEFINT A-Z

REM $INCLUDE: 'QBX.BI'               'Use QBX.BI instead if using PDS

DECLARE SUB DefineChar (BitMask$, Ascii%, TableNum%)
DECLARE SUB LoadDefaultFont ()
DECLARE FUNCTION MonitorType% ()
DECLARE SUB TwoFontsOff ()
DECLARE SUB TwoFontsOn ()

DIM SHARED Reg AS RegTypeX          'For the InterruptX routine

DEFSNG A-Z
SUB DefineChar (BitMask$, Ascii%, TableNum%)

	'Redefines the ASCII character "Ascii%" in CGT "TableNum%" using
	'the "BitMask$"

	STATIC BytesPerChar%

	IF BytesPerChar% = 0 THEN           'If we haven't set this yet..
		SELECT CASE MonitorType%        'What type of display?
			CASE -1                     'If VGA
				BytesPerChar% = 16      '16 scan lines (1 byte each)

			CASE -2                     'If EGA
				BytesPerChar% = 14      '14 scan lines

			CASE ELSE                   'If other, exit without action
				EXIT SUB
		END SELECT
	END IF

	Reg.ax = &H1100                 'Load custom charater
	Reg.bx = BytesPerChar% * 256    'Bytes per character into BH
	Reg.bx = Reg.bx + TableNum%     'Table number into BL
	Reg.cx = 1                      'Load single character
	Reg.dx = Ascii%                 'Into the position for this code

	DIM Temp AS STRING * 16         'Create a fixed-length string
	Temp = BitMask$                 'So we can use the same code for
									' QB4.x and PDS compilers.

	Reg.es = VARSEG(Temp)           'TempBitMap's segment into ES
	Reg.bp = VARPTR(Temp)           'Its offset into BP
	CALL InterruptX(&H10, Reg, Reg) 'Call the BIOS

END SUB

SUB LoadDefaultFont

	'Loads the default display CGT from EGA/VGA adapters' ROM. Use
	'this routine before you exit your program to reset the default
	'CGT back to its start-up values.

	SELECT CASE MonitorType%
		CASE -1, -2                         'Let BIOS do the hard part
			Reg.ax = &HE                    'Load the default CGT
			CALL InterruptX(&H10, Reg, Reg) ' by switching into
			Reg.ax = 3                      ' graphics mode and
			CALL InterruptX(&H10, Reg, Reg) ' back again.

		CASE ELSE                           'If not an EGA or VGA
			EXIT SUB                        ' color monitor, exit.
	END SELECT

END SUB

FUNCTION MonitorType%

	'This routine determines if an EGA or VGA display adapter is
	'present. Returns -1 if a VGA Color adapter is detected, -2 if
	'an EGA Color adapter is detected, or 0 if neither are detected.

	STATIC CheckedMonitor%, MonType%

	IF CheckedMonitor% THEN             'If we've been here once,
		MonitorType% = MonType%         ' no need to go through this
		EXIT FUNCTION                   ' again.
	END IF

	Reg.ax = &H1A00                     'Get "combination code"
	CALL InterruptX(&H10, Reg, Reg)     ' A VGA specific function
	IF Reg.ax <> &H1A00 THEN            'VGA was found
		IF (Reg.bx AND &HFF) = 8 THEN   'Is a VGA Color monitor there?
			MonType% = -1               'Yes!
		END IF
	ELSE                                'If VGA not present, check EGA
		Reg.ax = &H1200                 'Get EGA/VGA Info
		Reg.bx = &H10                   'Subfunction 10h
		CALL InterruptX(&H10, Reg, Reg)
		IF (Reg.bx AND &HFF00) = 0 AND Reg.bx <> &H10 THEN
			MonType% = -2               'An EGA Color card was found
		END IF
	END IF
	CheckedMonitor% = -1                'So we don't have to execute
	MonitorType% = MonType%             ' this code a second time.

END FUNCTION

SUB TwoFontsOff

	'This routine disables CGT1 and re-enables HI color attributes.

	Reg.ax = &H1103                     'Select active CGT
	Reg.bx = 0                          'Use only CGT0
	CALL InterruptX(&H10, Reg, Reg)     'Call Interrupt 10h
	FOR Attr% = 8 TO 15                 'Re-enable the display of HI
		PALETTE Attr%, Attr%            ' attributes.
	NEXT

END SUB

SUB TwoFontsOn

	'Uses the video BIOS to simultaneously enable character generator
	'tables (CGT) 0 and 1, allowing up to 512 different characters on
	'the screen. The high-intensity color attribute (HI) is used to
	'determine from which CGT a character's bit map is fetched. When
	'HI is off (foreground colors of 0-7 are used), the characters
	'are fetched from CGT0. When HI is on (foreground colors 8 thru
	'15, and 24 thru 31), the character's bit map is fetched from
	'CGT1.

	Reg.ax = &H1103                     'Select active CGT
	Reg.bx = 4                          'Make CGT0 and CGT1 active
	CALL InterruptX(&H10, Reg, Reg)     'Call Interrupt 10h
	FOR Attr% = 8 TO 15                 'Disable the display of HI
		PALETTE Attr%, Attr% - 8        ' attributes.
	NEXT

END SUB

