'written by Anton Monroe
'Last worked on: June 21, 1993
'
'QB2PB converts QuickBasic source code into PowerBasic code by reading the
'QB file, translating it line by line and writing the output to a temporary
'file called TempFile$.  Things that need to be at the beginning of the file
'go into another temporary file called OutFile$.  The two files are then
'merged.
'
'If you are interested in modifying anything, here's a rough outline of the
'strategy:
'
'   Subs BreakUpLine and UnMultiply break up a line into its significant
'   parts.  An array called Word$() is used to hold each word or symbol.
'   Changes are made by inserting, deleting or modifying
'   elements in the array.
'
'   If it's a $include line, Sub ReadIncludeFile will read the specified
'   file for DEFxxx type statements and constant declarations.
'
'   Sub MakeFunctionChanges makes changes to elements that could be anywhere
'   in the line, like functions, constants, and variables.  It goes through
'   the array looking at one word at a time.  If the word is one of the
'   functions that are not compatible, it is replaced it with the PB
'   equivalent.  If the word is in a list of declared constants, it is
'   changed to a PB equate.  If the word is in a list of variables declared
'   as RegType, it is changed to the PB Reg form by calling ReplaceRegs.
'
'   Sub MakeCommandChanges identifies problem lines based on the first word
'   of the array, ie, statements rather than functions.  Most changes are
'   made within the routine, but DIM and CONST are handled by calling
'   ProcessDim and ChangeConstants.  In the case of one-line IF/THEN
'   statements, MakeCommandChanges calls itself to process part of the
'   line.
'
'   Sub RebuildLine combines the elements of the array back into a line that
'   PB will (hopefully) understand.
'
'   Sub PrintToFile prints the new line to TempFile$.
'
'   When done processing, Sub CombineFiles combines OutFile$ and TempFile$.
'
'Some important variables:
'
'   Word$()         :the array that holds the contents of the line being
'                    processed.  Upper bound is 100.
'   Word$(WordCtr%) :whatever word is currently being examined.
'   LastWord%       :tells a routine when to quit if I only want it to
'                    process a part of the Word$() array.  If you delete or
'                    insert an element, be sure to decr or incr LastWord%.
'   RegVariables$() :contains the names of any variables declared as RegType.
'   DefType$()      :stores the default variable types for a-z.
'   PrintOldLines%  :flag that tells it whether to show original lines.
'   ChangeRegType%  :flag that tells whether to convert RegType variables.
'   OutFile$        :the first part of the new file.
'   TempFile$       :the second part of the new file.
'   ThisLine$       :the current line from the old file
'   LineChanged%    :flag to show if the current line has been changed.  If
'                    you make any changes to elements of Word$(), set this
'                    flag.
'   Const$()        :an array holding the old and new names of constants.
'                    upper bound is 300.
'   ConstCtr%       :the number of constants declared.
'
'----------------------------------------------------------------------------

   DEFINT A-Z

$INCLUDE "lineinp.bas"
$INCLUDE "comline.bas"
$INCLUDE "basline.bas"

   SHARED CRLF$
   %False = 0
   %True = -1
   CRLF$ = chr$(&h0D, &h0A)
   REDIM Word$(100)
   REDIM Args$(0)
   DIM RegVariable$(4)
   DIM DefType$(26)

'We need to keep track of any DEFxxx statements, because there may be DIM
'statements that do not specify what type of variable they are declaring.
'Default type is Single.
   FOR n% = 1 to 26
      DefType$(n%) = "!"
   NEXT n%
'Defaults that can be changed by command-line switches:
   PrintOldLines% = %False
   ChangeRegType% = %True

'Args$() contains the command line parameters
   CALL Comline (NumArgs%, Args$())
'Test for no file name
   IF NumArgs% = 0 THEN ShowSyntax
   array scan Args$(1), collate ucase, = "/OLDLINES", to Hit%
   IF istrue Hit% THEN
      PrintOldLines% = %True
      array delete Args$(Hit%)
   END IF
   array scan Args$(1), collate ucase, = "/NOREGTYPE", to Hit%
   IF istrue Hit% THEN
      ChangeRegType% = %False
      array delete Args$(Hit%)
   END IF
'Test for wildcard or unrecognized switch
   FOR Ctr% = 1 to NumArgs%
      IF instr(Args$(Ctr%), any "/*?") THEN
         print
         print "INVALID PARAMETER: "; Args$(Ctr%)
         CALL ShowSyntax
      END IF
   NEXT Ctr%
   InFile$ = Args$(1)

   array scan Args$(1), collate ucase, = "TO", to Hit%
   IF istrue Hit% THEN
      OutFile$ = Args$(Hit% + 1)
   ELSE
'use default OutFile$-- the same name + ".q2p"
      OutFile$ = extract$(InFile$, ".")
      OutFile$ = OutFile$ + ".q2p"
   END IF

   IF dir$(InFile$) = "" THEN
      print "Cannot find "; InFile$
      END
   ELSEIF dir$(OutFile$) <> "" THEN
      print OutFile$; " already exists.  Overwrite it? "; chr$(174) + "Yes" + chr$(175);
      locate, (pos - 3), 1
      WHILE not instat : WEND
      Ky$ = inkey$
      SELECT CASE Ky$
      CASE <> chr$(13), "Y", "y"
         END
      END SELECT
   END IF
   print
   print "Converting "; InFile$; " to "; OutFile$

   CALL CheckForQuickSavedFile

'allow up to 300 QB constants to be changed
   REDIM Const$(0 to 300, 1 to 2)

   TempFile$ = environ$("TEMP") + "qb2pb.$$$"
   TempFile% = freefile
   OPEN TempFile$ FOR output as TempFile%
'open new file for lines that need to be added at top of new file:
   OutFile% = freefile
   OPEN OutFile$ FOR output as OutFile%
   print #OutFile%, "'Converted by QB2PB from "; ucase$(InFile$)
   print #OutFile%, "'"; date$; "  "; left$(time$, 5)
   print #OutFile%,
   CALL ListMAKFile

   DO
'Remainder$ is the unprocessed remainder of a multiple-command line.
'null out OldLine$ here to keep it from printing the old line
'every time it prints part of a multiple-command line.
      IF Remainder$ <> "" THEN
         ThisLine$ = Remainder$
         Remainder$ = ""
      ELSE
'null out Comment$ and get the next line from the input file
         Comment$ = ""
         LastWord% = 0
         ThisLine$ = LineInput$(InFile$, Done%)
'display the number of lines read from the old file on screen
         incr FileLineCtr%
         locate, 1, 0
         print "Converting Line"; FileLineCtr%;
      END IF                 'end of is there a Remainder$

'if it's a multiple-command line, I am going to separate it.  Remainder$ is
'the remainder after the ":".  It will be processed on the next loop.
      CALL UnMultiply (ThisLine$, Remainder$)

'break up the command line into words.  Indent$ is the spaces at the
'beginning, Comment$ is any comment.
      CALL BreakUpLine (ThisLine$, Indent$, Word$(), Comment$, LastWord%, OldLine$)

'if there is nothing there, go to ELSE to look at the comment tail
      IF LastWord% > 0 THEN
'change a couple of other things that involve replacing strings.  It's
'easier to do now, before MakeCommandChanges gets ahold of the line and
'starts messing up the word spacing.
         MakeFunctionChanges
'a lot of changes are based on the command in the first word.  They are in
'a separate sub:
         MakeCommandChanges 1, LastWord%
      ELSE
'statement part of line is blank, look at comment part.  If it is a
'metacommand, take off the apostrophe.  If it is a $include line, change it.
         IF mid$(Comment$, 2, 1) = "$" THEN Comment$ = mid$(Comment$, 2)
         IF ucase$(left$(Comment$, 9)) = "$INCLUDE:" THEN
'if include for QB.BI, delete it.  Otherwise, change punctuation.
            IF ucase$(mid$(Comment$, 11, 7)) = "'QB.BI'" THEN
               Comment$ = ""
               LineChanged% = %True
               print #TempFile%, "'deleted QB.BI include file:"
            ELSE
'get the name of the included file:
               IncludeName$ = mid$(Comment$, instr(Comment$, "'"))
               IncludeName$ = remove$(IncludeName$, any " '")

'read the include file for CONST OR DEFxxx statements.  First close out
'LineInput by CALLing it-- so the value of ThisFile$ is not changed.
               Done% = %True
               CALL LineInput$(InFile$, Done%)
               Done% = %False
               CALL ReadIncludeFile (IncludeName$)

'PB doesn't need a path for include files, and it probably wouldn't be right
'anyway, so remove it:
               DO
                  BackslashPos% = instr(IncludeName$, "\")
                  IF istrue BackslashPos% THEN
                     IncludeName$ = mid$(IncludeName$, BackslashPos% + 1)
                  ELSE
                     EXIT LOOP
                  END IF
               LOOP
'and rewrite the line.  Of course, it's not a comment any more, but in this
'case it won't make a difference.
               Comment$ = "$INCLUDE " + chr$(34) + IncludeName$ + chr$(34)
               LineChanged = %True

'find our place again in the main file.
               FOR Ctr% = 1 to FileLineCtr%
                  CALL LineInput$(InFile$, Done%)
               NEXT Ctr%
            END IF         'end of Does it refer to QB.BI?
         END IF            'end of Is it a $include line
      END IF               'end "if not a blank line"
'Changes have all been made.  Rebuild new line and print it.
      CALL RebuildLine (ThisLine$, Indent$, Word$(), Comment$, LastWord%)
      CALL PrintToFile (ThisLine$)
   LOOP UNTIL istrue Done%

   CLOSE
   CALL CombineFiles
   print
   print "Done"
   END

'**********************************************************
'*  SUBROUTINES                                           *
'**********************************************************

SUB PrintToFile (LineToPrint$)
'-----------------------------------------------------------------------------
'This procedure prints to the output file the string that is passed to it.
'if the line has been changed, it prints the old line.  The comments are
'printed as a separate line.
'
'Parameters:  LineToPrint$ is the line to print
'
'-----------------------------------------------------------------------------

   SHARED OldLine$, PrintOldLines%, LineChanged%, TempFile%

'if user wants and the line was changed, show the original line as a comment.
'Reset LineChanged% flag when done.
   IF istrue PrintOldLines% THEN
      IF istrue LineChanged% THEN
         print #TempFile%, OldLine$
      END IF
   END IF
   print #TempFile%, LineToPrint$
   LineChanged% = %False
END SUB

SUB ShowSyntax
'-----------------------------------------------------------------------------
'Tells how to use it.
'
'-----------------------------------------------------------------------------

   print
   print "USE:      QB2PB [switches] (Input file) [TO (Output file)] [switches]
   print "          Converts a QuickBasic file to PowerBasic."
   print "          Source file must be an ASCII file."
   print "          If destination file name is not given, destination is <filename>.Q2P
   print "Switches:
   print "          /noregtype     -do not change RegType variables
   print "          /oldlines      -show original of any changed lines
   END
END SUB

SUB ReplaceRegs
'-----------------------------------------------------------------------------
'RegVariable$() is an array that holds the names of any variables DIMmed as
'RegType or RegTypeX and saved by Sub ProcessDim.  Change the name of the
'variable to Reg(n).  If it should be "Reg, n = x", it will be changed again
'by MakeCommandChanges.  It's easier to change it twice than to figure out
'which it should be here.
'
'-----------------------------------------------------------------------------

   SHARED RegVariable$(), RegVarCtr%, Word$(), LastWord%, LineChanged%
   SHARED ThisLine$

   FOR Ctr% = 0 to (RegVarCtr% - 1)
      ThisReg$ = ucase$(RegVariable$(Ctr%))
'check ThisLine$ for the RegVariable first.  If it's not there, save time
'by not searching word by word.
      IF instr(ucase$(ThisLine$), ThisReg$ + ".") > 0 THEN
         WordCtr% = 1
         WHILE WordCtr% <= LastWord%
            IF extract$(trim$(Word$(WordCtr%)), ".") = ThisReg$ THEN
'uppercase the word to make it easier to change.
               Word$(WordCtr%) = ucase$(Word$(WordCtr%))
'figure out which register this is:
               Register$ = rtrim$(mid$(Word$(WordCtr%), instr(Word$(WordCtr%), ".") + 1))
               Word$(WordCtr%) = "Reg"
'insert "(", Register$, and ")" as separate words so it can be analyzed by
'MakeCommandChanges
               array insert Word$(WordCtr% + 1), ")"
               SELECT CASE Register$
               CASE "FLAGS"
                  array insert Word$(WordCtr% + 1), "0"
               CASE "AX"
                  array insert Word$(WordCtr% + 1), "1"
               CASE "BX"
                  array insert Word$(WordCtr% + 1), "2"
               CASE "CX"
                  array insert Word$(WordCtr% + 1), "3"
               CASE "DX"
                  array insert Word$(WordCtr% + 1), "4"
               CASE "SI"
                  array insert Word$(WordCtr% + 1), "5"
               CASE "DI"
                  array insert Word$(WordCtr% + 1), "6"
               CASE "BP"
                  array insert Word$(WordCtr% + 1), "7"
               CASE "DS"
                  array insert Word$(WordCtr% + 1), "8"
               CASE "ES"
                  array insert Word$(WordCtr% + 1), "9"
               END SELECT
               array insert Word$(WordCtr% + 1), "("
               incr LastWord%, 3
               LineChanged% = %True
'no point in checking what we just inserted:
               incr WordCtr%, 3
            END IF         'end of is ThisReg$ in this word
            incr WordCtr%  'go to next word
         WEND
      END IF               'end of is ThisReg$ in this line
   NEXT Ctr%               'go to next RegVariable$
END SUB

SUB ProcessDIM (WordCtr%, LastWord%)
'-----------------------------------------------------------------------------
'This procedure processes a DIM statement to convert something like:
'    DIM SHARED Var1 as VariableType, Var2 as VariableType, ...
'into:
'    DIM Var1 as shared VariableType, Var2 as shared VariableType, ...
'
'It also deletes any DIM statements that declare a variable as RegType or
'RegTypeX, since that is not needed in PB.
'
'It saves the name of any RegType/RegTypeX variables in RegVariables$()
'so they can be replaced elsewhere in the source file by Sub ReplaceRegs.
'
'-----------------------------------------------------------------------------

   SHARED RegVariable$(), RegVarCtr%, Word$()
   SHARED ChangeRegType%, LineChanged%, DefType$()

   DimWord$ = rtrim$(Word$(WordCtr%))
   IF Trim$(Word$(WordCtr% + 1)) = "SHARED" THEN
      SharedFlag% = %True
'delete "shared", it will have to be reinserted elsewhere
      array delete Word$(WordCtr% + 1)
      decr LastWord%
      LineChanged% = %True
   END IF

'Part 1
'Variables that show registers (Regs, InRegs, OutRegs) will be replaced
'with "Reg(1)" etc., unless user has turned off ChangeRegType%. So look
'for a "DIM xxx AS RegType", save the variable name in an array for later,
'and blank out the statement by deleting "xxx AS RegType".

   IF ChangeRegType% = %True THEN
      FOR Ctr% = LastWord% to WordCtr% step -1
         SELECT CASE Trim$(Word$(Ctr%))
         CASE "REGTYPE", "REGTYPEX"
'for instance, in "DIM Regs AS RegType", if it finds "RegType" in the third
'word, the variable name we want to save is "Regs".
            RegVariable$(RegVarCtr%) = ltrim$(rtrim$(Word$(Ctr% - 2)))
'get ready for the next RegType variable
            incr RegVarCtr%
'delete "Regs as RegType"
            array delete Word$(Ctr% - 2)
            array delete Word$(Ctr% - 2)
            array delete Word$(Ctr% - 2)
            decr LastWord%, 3
            decr Ctr%, 3
'now Word$(Ctr%) is either DimWord$ or a comma.  Delete the comma and add a
'space to the word before it.
            IF Word$(Ctr%) = "," THEN
               array delete Word$(Ctr%)
               decr LastWord%
               Word$(Ctr% - 1) = Word$(Ctr% - 1) + " "
            END IF
            LineChanged% = %True
         END SELECT
      NEXT Ctr%
'we may have deleted all the declarations and just have a "DIM" left.  If
'so, delete it.
      IF WordCtr% = LastWord% THEN
         array delete Word$(WordCtr%)
         decr LastWord%
      END IF
   END IF           'end of if ChangeRegType% = %True


'Part 2:
'if it's a DIM SHARED statement, it needs to be changed.
'PB 3.00b will accept "DIM Var$(10)" but not "DIM Var$(10) AS SHARED".  I
'hope that gets fixed someday, but until then, take out the type id
'character and spell out the type.
   IF istrue SharedFlag% THEN
      Ctr% = WordCtr% + 1
'-- Word$(Ctr%) is the variable name
      DO UNTIL Ctr% > LastWord%
         SELECT CASE ucase$(rtrim$(Word$(Ctr% - 1)))
         CASE DimWord$, ","
'found a variable name.
            SELECT CASE right$(trim$(Word$(Ctr%)), 1)
            CASE "%"
               VariableType$ = "INTEGER"
            CASE "&"
               VariableType$ = "LONG"
            CASE "!"
               VariableType$ = "SINGLE"
            CASE "#"
               VariableType$ = "DOUBLE"
            CASE "$"
               VariableType$ = "STRING"
            CASE ELSE
'if none of the above, use the default variable type
               DefaultType$ = DefType$(asc(left$(trim$(Word$(Ctr%)), 1)) - 64)
               SELECT CASE DefType$(asc(left$(trim$(Word$(Ctr%)), 1)) - 64)
               CASE "%"
                  VariableType$ = "INTEGER"
               CASE "&"
                  VariableType$ = "LONG"
               CASE "!"
                  VariableType$ = "SINGLE"
               CASE "#"
                  VariableType$ = "DOUBLE"
               CASE "$"
                  VariableType$ = "STRING"
               END SELECT
            END SELECT
'remove the type id:
            Word$(Ctr%) = remove$(Word$(Ctr%), any "%&!#$")
'If it's an array, fast forward to the end of the parentheses:
            IF Word$(Ctr% + 1) = "(" THEN
               WHILE Word$(Ctr%) <> ")"
                  incr Ctr%
               WEND
            END IF
'If there is not already an "as", insert one and a variable type
            IF Trim$(Word$(Ctr% + 1)) <> "AS" THEN
               array insert Word$(Ctr% + 1), " AS "
               incr LastWord%
'insert the type after the "as"
               array insert Word$(Ctr% + 2), VariableType$
               incr LastWord%
            END IF
'insert "shared"
            array insert Word$(Ctr% + 2), "SHARED "
            incr LastWord%
'it's safe to skip over some words here.
            incr Ctr%, 3
         END SELECT
         incr Ctr%
      LOOP
   END IF                'end of is it shared

END SUB

SUB MakeCommandChanges (WordCtr%, LastWord%)
'-----------------------------------------------------------------------------
'This sub makes changes based on a command. Normally that is the first word
'of the line, but in a few cases, it will call itself to examine part of a
'line.  For instance, in "IF function THEN statement2 ELSE statement3",
'it will have to look at the statements after THEN and ELSE to see if they
'need to be changed.  WordCtr% tells it which word of the line to start at
'and LastWord% which word to end at.
'
'Calls:  MakeCommandChanges
'        ChangeConstants       :for CONST statements
'        ProcessDim            :for DIM statements
'        Trim$                 :same as "ucase$(rtrim$(ltrim$"
'-----------------------------------------------------------------------------

   SHARED Word$(), Indent$, InSub%, LineChanged%
   SHARED TempFile%, OutFile%, DefType$()
   STATIC BinaryBase1%

'Take off the space at the end of the word before comparing:
   SELECT CASE Trim$(Word$(WordCtr%))

   CASE "CALL"
'then next word may be "interrupt".  No need for a temporary last word here,
'because it's not changing what part of the array to process.
      CALL MakeCommandChanges ((WordCtr% + 1), LastWord%)

   CASE "CASE"
'change "CASE IS = " to "CASE = "
      IF ucase$(Word$(WordCtr% + 1)) = "IS " THEN
         array delete Word$(WordCtr% + 1)
         decr LastWord%
         LineChanged% = %True
      END IF

   CASE "CLS"
'QB uses numbers to modify CLS, PB uses words
      SELECT CASE rtrim$(Word$(WordCtr% + 1))
      CASE "0"
         replace "0" with "SCREEN" in Word$(WordCtr% + 1)
         LineChanged% = %True
      CASE "1"
         replace "1" with "GRAPHICS" in Word$(WordCtr% + 1)
         LineChanged% = %True
      CASE "2"
         replace "2" with "TEXT" in Word$(WordCtr% + 1)
         LineChanged% = %True
      END SELECT

   CASE "CLEAR"
'the comma shows that clear is used for stack, so fix it:
      IF Word$(WordCtr% + 1) = "," THEN
         StackSize$ = Trim$(Word$(WordCtr% + 3))
         print #TempFile%, "'Added $STACK to set stack size.  You may want to delete CLEAR."
         print #OutFile%, "   $STACK "; StackSize$
         print #OutFile%,
         FOR n% = 1 to 3
            array delete Word$(WordCtr% + 1)
         NEXT n%
         decr LastWord%, 3
         LineChanged% = %True
      END IF

   CASE "CONST"
'delete "CONST"
      array delete Word$(WordCtr%)
      decr LastWord%
      LineChanged% = %True
'change QB "CONST a% =" to PB "%a =". Pass WordCtr% by copy so it won't
'be changed
      CALL ChangeConstants ((WordCtr%), Word$(), LastWord%)
'trick PrintToFile into making separate lines by turning commas into CRLF
'and the following blankspace into Indent$
      FOR Ctr% = WordCtr% to LastWord%
         IF Word$(Ctr%) = "," THEN
            Word$(Ctr%) = CRLF$
            Word$(Ctr% + 1) = Indent$ + ltrim$(Word$(Ctr% + 1))
         END IF
      NEXT Ctr%

   CASE "DEFINT", "DEFSNG", "DEFDBL", "DEFLNG", "DEFSTR"
'record default variable types in array for use in DIM statements later.  The
'array DefType$() contains 26 elements showing the default types.  That is,
' "DEFINT A-D" would make elements 1-4 = "%".
      SELECT CASE Trim$(Word$(WordCtr%))
      CASE "DEFINT"
         VarSymbol$ = "%"
      CASE "DEFSNG"
         VarSymbol$ = "!"
      CASE "DEFDBL"
         VarSymbol$ = "#"
      CASE "DEFLNG"
         VarSymbol$ = "&"
      CASE "DEFSTR"
         VarSymbol$ = "$"
      END SELECT
      BeginLetter% = asc(trim$(Word$(WordCtr% + 1))) - 64
      IF Trim$(Word$(WordCtr% + 2)) = "-" THEN
         EndLetter% = asc(trim$(Word$(WordCtr% + 3))) - 64
      ELSE
         EndLetter% = BeginLetter%
      END IF
      FOR Letter% = BeginLetter% to EndLetter%
         DefType$(Letter%) = VarSymbol$
      NEXT Letter%

   CASE "DIM", "REDIM"
'analyzing a DIM statement involves a lot.  Put it in a separate sub just to
'keep things manageable.
      CALL ProcessDIM ((WordCtr%), LastWord%)

   CASE "ELSE"
'it only hits here if "ELSE" is the first word.  One-line IF/THEN structures
'are handled under "CASE "IF", "THEN"".
'PowerBasic needs the ELSE to be on its own line; if there is anything but
'spaces after that, split it into two lines by inserting a CR/LF.  The
'indent for the next line won't be right, but who cares?
'remove tabs in case there are any after ELSE and before a comment
      IF ltrim$(Word$(WordCtr% + 1), any chr$(9) + " ") <> "" THEN
         LineChanged% = %True
         Word$(WordCtr%) = Word$(WordCtr%) + CRLF$ + Indent$
'process everything after "ELSE":
         CALL MakeCommandChanges ((WordCtr% + 1), LastWord%)
      END IF

   CASE "END"
      SELECT CASE ucase$(rtrim$(Word$(WordCtr% + 1)))
      CASE "FUNCTION", "SUB"
'These are unused.  I was going to do something with them, but haven't got
'around to it.
         InSub% = %False
         ThisSub$ = ""
      END SELECT

   CASE "IF", "THEN", "ELSEIF"
'change "if not" to "if isfalse", just because I understand it better.
      IF ucase$(rtrim$(Word$(WordCtr% + 1))) = "NOT" THEN
         replace "NOT" with "ISFALSE" in Word$(WordCtr% + 1)
         LineChanged% = %True
      END IF
'if it is a one-line IF/THEN statement, process each part as if it were a
'separate line:
      TempLastWord% = LastWord%
      FOR Ctr% = WordCtr% to LastWord%
         SELECT CASE Trim$(Word$(Ctr%))
         CASE "THEN"
            BeginWord% = Ctr% + 1
         CASE "ELSE"
            TempLastWord% = Ctr% - 1
            EXIT FOR
         END SELECT
      NEXT Ctr%
'if the THEN is not at the end of the line, do from there to end of line, or
'to the ELSE if it exists:
      IF BeginWord% =< LastWord% THEN
         CALL MakeCommandChanges (BeginWord%, TempLastWord%)
'if array delete or insert was used in the called routine, update the
'value of LastWord%
         Difference% = (Ctr% - 1) - TempLastWord%
         decr LastWord%, Difference%
'if there is an ELSE, do from there to end of line
         IF TempLastWord% < LastWord% THEN
            BeginWord% = TempLastWord% + 2
            CALL MakeCommandChanges (BeginWord%, LastWord%)
         END IF
      END IF

   CASE "INTERRUPT", "INTERRUPTX", "INTERRUPT(", "INTERRUPTX("
'delete left parenthesis, comma, and "InRegs, OutRegs)"
'PB needs "CALL" in front of INTERRUPT:
      IF ucase$(Word$(WordCtr% - 1)) <> "CALL " THEN
         array insert Word$(WordCtr%), "CALL "
         incr LastWord%
         incr WordCtr%
      END IF
      Word$(WordCtr%) = "INTERRUPT "
'if it finds a left parenthesis, delete it.  Skip the next word which is
'the Interrupt number, and delete everything through the right parenthesis.
      IF Word$(WordCtr% + 1) = "(" THEN
         array delete Word$(WordCtr% + 1)
         decr LastWord%
         DO
            ThisWord$ = Word$(WordCtr% + 2)
            array delete Word$(WordCtr% + 2)
            decr LastWord%
         LOOP UNTIL ThisWord$ = ")"
      ELSE
'keep the Interrupt number; delete the RegType variables
         FOR n% = 1 to 4
            array delete Word$(WordCtr% + 2)
            decr LastWord%
         NEXT n%
      END IF
      LineChanged% = %True

   CASE "OPEN"
'if open something for binary, add "Option Binary Base 1" to force PB to use
'QB's method of numbering binary file positions.  BinaryBase1% is a flag to
'keep it from doing it more than once.
      FOR Ctr% = WordCtr% to LastWord%
         IF Trim$(Word$(Ctr%)) = "BINARY" THEN
            IF isfalse BinaryBase1% THEN
               print #OutFile%, 'LINE ADDED:"
               print #OutFile%, "OPTION BINARY BASE 1"
               BinaryBase1% = %True
            END IF
            EXIT FOR
         END IF
      NEXT Ctr%

   CASE "REG"
'the "Reg(x) = y" needs to be changed to the statement "Reg x, y" by adding
'a blank to "Reg", deleting "(", deleting ")", and deleting "= "
      Word$(WordCtr%) = Word$(WordCtr%) + " "
      array delete Word$(WordCtr% + 1)
      array delete Word$(WordCtr% + 2)
      decr LastWord%, 2
      Word$(WordCtr% + 2) = ","
      Word$(WordCtr% + 3) = " " + ltrim$(Word$(WordCtr% + 3))

   CASE "VIEW"
      SELECT CASE Trim$(Word$(WordCtr% + 1))
      CASE "PRINT"
'change "view print" to "view text"
         replace "PRINT" with "TEXT" in Word$(WordCtr% + 1)
         IF LastWord% > WordCtr% + 1 THEN
            IF Trim$(Word$(WordCtr% + 3)) = "TO" THEN
'view print does not give columns.  tell PB to use the first and last columns.
               ULCorner$ = "(1," + rtrim$(Word$(WordCtr% + 2)) + ")"
               LRCorner$ = "(pbvScrnCols," + rtrim$(Word$(WordCtr% + 4)) + ")"
'delete "TopRow TO BottomRow"
               array delete Word$(WordCtr% + 2)
               array delete Word$(WordCtr% + 2)
               array delete Word$(WordCtr% + 2)
               decr LastWord%, 3
'insert "(1,TopRow)-(BottomRow,pbvScrnCols)"
               array insert Word$(WordCtr% + 2), UlCorner$ + "-" + LRCorner$ + " "
               incr LastWord%
            END IF
         ELSE
'it's VIEW PRINT only, specify whole screen
            Word$(WordCtr% + 1) = rtrim$(Word$(WordCtr% + 1)) + " "
            array insert Word$(WordCtr% + 2), "(1,1)-(pbvScrnCols, pbvScrnRows) "
            incr LastWord%
         END IF
      END SELECT
      LineChanged% = %True

   CASE "SUB", "FUNCTION"
'For future use:
      ThisSub$ = Word$(WordCtr%) + Word$(WordCtr% + 1)
      InSub% = %True
   END SELECT
END SUB


SUB CheckForQuickSavedFile
'-----------------------------------------------------------------------------
'Make sure the input file was not saved in QB Quicksaved format.  It has to
'be an ASCII file.  I took this test from something else, it seems to work.
'
'-----------------------------------------------------------------------------

   SHARED InFile$

   InFile% = freefile
   OPEN InFile$ FOR BINARY as #InFile%
   get$ #InFile%, 1, Test$
   CLOSE #InFile%
   IF Test$ = chr$(252) THEN
      print InFile$ + " is not an ASCII file."
      END
   END IF
END SUB

SUB ListMAKFile
'-----------------------------------------------------------------------------
'Description: A convenience for the user.  It checks for the existence of a
'.mak file of the same name as the input file.  If it exists, the contents
'of the .mak file are listed as comments at the top of the new .q2p file.
'
'Calls: LineInput$
'
'-----------------------------------------------------------------------------
   SHARED BaseName$, OutFile%

'list contents of MAK file in case user is interested:
   MakFile$ = ucase$(BaseName$) + ".MAK"
   IF dir$(MakFile$) <> "" THEN
      print #OutFile%, "'Other modules listed in "; MakFile$; ":"
      DO
         MakLine$ = LineInput$(MakFile$, Done%)
         MakLine$ = rtrim$(ltrim$(MakLine$))
         print #OutFile%, "'    "; MakLine$
      LOOP UNTIL istrue Done%
      print #OutFile%,
   END IF
END SUB

SUB MakeFunctionChanges
'-----------------------------------------------------------------------------
'Makes changes (like functions, constants, etc.) that are not based on the
'command at the beginning of the line.  Since they can be anywhere in a line
'I just go through each word of the line and look for a string.
'
'Calls:   ReplaceRegs    :to change QB's RegType variables
'
'-----------------------------------------------------------------------------
   SHARED RegVarCtr%, Word$(), LastWord%, Const$(), ConstCtr%, LineChanged%

   FOR WordCtr% = 1 to LastWord%
      SELECT CASE Trim$(Word$(WordCtr%))
      CASE = "SADD"
         Word$(WordCtr%) = "STRPTR"
'I don't use these, but they are easy to change:
      CASE = "CVSMBF"
         Word$(WordCtr%) = "CVMS"
      CASE = "CVDMBF"
         Word$(WordCtr%) = "CVMD"
      CASE = "MKSMBF$"
         Word$(WordCtr%) = "MKMS$"
      CASE = "MKDMBF$"
         Word$(WordCtr%) = "MKMD$"
      CASE ELSE
'search array of constants to see if this is one:
         array scan Const$(1, 1) FOR ConstCtr%, collate ucase, = (trim$(Word$(WordCtr%))), to Hit%
         IF istrue Hit% THEN
'found a constant, change it:
            Word$(WordCtr%) = ucase$(Word$(WordCtr%))
            replace Const$(Hit%, 1) with Const$(Hit%, 2) in Word$(WordCtr%)
            LineChanged% = %True
         END IF
      END SELECT
   NEXT WordCtr%     'next word
'check for any variables DIMmed as RegType/RegTypeX and fix them.  Skip it if
'RegVarCtr% = 0, because there aren't any yet.
   IF RegVarCtr% > 0 THEN CALL ReplaceRegs
END SUB

SUB ChangeConstants (WordCtr%, Word$(), LastWord%)
'-----------------------------------------------------------------------------
'Processes a line that begins with "CONST".  Changes QB integer constants to
'PB by putting a % in front.  QB string constants are not allowed in PB, so
'it changes them to variables.  String constants in module-level code become
'shared and those in procedures do not, to be consistent with the scope of
'QB constants.  PB only allows 2-byte integer constants, if the value is
'over 32767, the user will have to figure out what to do.
'
'-----------------------------------------------------------------------------
   SHARED Indent$, InSub%, TempFile%
   SHARED Const$(), ConstCtr%

'look for a "= " to find the next constant
   DO
      WHILE Word$(WordCtr%) <> "= " and WordCtr% <= LastWord%
         incr WordCtr%
      WEND
      IF WordCtr% > LastWord% THEN EXIT LOOP
'Look for clues that it is a string constant, either quotes or a $ in the
'word after the "= ".  (This isn't quite foolproof.)  If it is, make it a
'shared variable by inserting a "shared" declaration, unless it is in a Sub.
      IF instr(Word$(WordCtr% + 1), any chr$(34) + "$") > 0 THEN
         IF isfalse InSub% THEN
            StringConstList$ = StringConstList$ + ltrim$(Word$(WordCtr% - 1))
         END IF
'otherwise, change by adding a % in front, delete any type declarations.
'Values larger than 32767 will cause an error in the PB IDE, user will have
'to decide what to do about that.
      ELSE
         incr ConstCtr%
'the old name; uppercase it to make replacing it easier:
         Const$(ConstCtr%, 1) = Trim$(Word$(WordCtr% - 1))
'the new name:
         Const$(ConstCtr%, 2) = "%" + remove$(ltrim$(rtrim$(Word$(WordCtr% - 1))), "%")
'replace it here:
         Word$(WordCtr% -1) = ucase$(Word$(WordCtr% -1))
         replace Const$(ConstCtr%, 1) with Const$(ConstCtr%, 2) in Word$(WordCtr% - 1)
      END IF
      incr WordCtr%
   LOOP
'trim the list of string constants, if any, put in commas, and print to
'new file
   IF StringConstList$ <> "" THEN
      StringConstList$ = rtrim$(StringConstList$)
      replace " " with ", " in StringConstList$
      print #TempFile%, "'QB String constants converted to PB shared variables:"
      print #TempFile, Indent$ + "SHARED " + StringConstList$
   END IF
END SUB

SUB CombineFiles
'-----------------------------------------------------------------------------
'It's done processing the input file.  Most changes have been written to a
'temporary file, and a few things have been written directly to the new .q2p
'file because they need to be at the beginning of it.  Now append the
'temporary file to the new one.
'
'-----------------------------------------------------------------------------
   SHARED TempFile$, OutFile$

   OutFile% = freefile
   OPEN OutFile$ FOR BINARY as OutFile%
   seek #OutFile%, lof(OutFile%)
   TempFile% = freefile
   OPEN TempFile$ FOR BINARY as TempFile%
   WHILE not eof(TempFile%)
      get$ #TempFile%, 4096, BufferString$
      put$ #OutFile%, BufferString$
   WEND
   CLOSE
   KILL TempFile$
END SUB

SUB ReadIncludeFile(IncludeFile$)
'-----------------------------------------------------------------------------
'Reads an Include file to see if any constants are declared in it or if
'there are any DEFxxx statements.  Saves them in the constant array just
'like ChangeConstants does.  A bug that may need to be fixed: This is case
'sensitive.  QB makes sure that all constants within a file are capitalized
'the same, but if you have %true in an include file and %True in the main
'file, QB2PB will not know they are the same thing.  IncludeFile$ is the
'name of the file to read.  Some of this duplicates code in other places,
'but it's easier this way.
'
'Calls:        ChangeConstants    :to read CONST statements
'              LineInput          :to read file
'              ReadIncludeFile    :if there is a $include file within a
'                                  $include file
'-----------------------------------------------------------------------------
'Indent$ is shared because I want the lines printed according to the
'indenting of the main file.  Others are local because there are variables by
'the same names waiting in the calling routine that I don't want overwritten.

   SHARED Indent$, DefType$()
   DIM ThisLine as LOCAL string, Comment as LOCAL string
   DIM OldLine as LOCAL string, NextLine as LOCAL string
   DIM Done as LOCAL integer
   REDIM Word(100) as LOCAL string

'if can't find the file, print message and keep going:
   IF dir$(IncludeFile$) = "" THEN
      locate, 1, 0
      print "Cannot find file: "; IncludeFile$
      EXIT SUB
   END IF

   DO
'Remainder$ is the unprocessed remainder of a multiple-command line.
      IF Remainder$ <> "" THEN
         ThisLine$ = Remainder$
         Remainder$ = ""
      ELSE
'null out Comment$ and get the next line from the input file
         Comment$ = ""
         LastWord% = 0
         ThisLine$ = LineInput$(IncludeFile$, Done%)
'display a message:
         incr LineCtr%
         locate, 1, 0
         print "Reading "; IncludeFile$; " Line: "; LineCtr%;
      END IF                 'end of is there a Remainder$

'if it's a multiple-command line, get the first part:
      CALL UnMultiply (ThisLine$, Remainder$)

'break up the command line into words:
      CALL BreakUpLine (ThisLine$, Indent$, Word$(), Comment$, LastWord%, OldLine$)

      IF LastWord% > 0 THEN
         WordCtr% = 1
         SELECT CASE Trim$(Word$(WordCtr%))
         CASE "CONST"
'delete "CONST"
            array delete Word$(WordCtr%)
            decr LastWord%
'change QB "CONST a% = " to PB "%a = "
            CALL ChangeConstants ((WordCtr%), Word$(), LastWord%)
         CASE "DEFINT", "DEFSNG", "DEFDBL", "DEFLNG", "DEFSTR"
'record default variable types in array for use in DIM statements later
            SELECT CASE Trim$(Word$(WordCtr%))
            CASE "DEFINT"
               VarSymbol$ = "%"
            CASE "DEFSNG"
               VarSymbol$ = "&"
            CASE "DEFDBL"
               VarSymbol$ = "!"
            CASE "DEFLNG"
               VarSymbol$ = "#"
            CASE "DEFSTR"
               VarSymbol$ = "$"
            END SELECT
            BeginLetter% = asc(trim$(Word$(WordCtr% + 1))) - 64
            IF Trim$(Word$(WordCtr% + 2)) = "-" THEN
               EndLetter% = asc(trim$(Word$(WordCtr% + 3))) - 64
            ELSE
               EndLetter% = BeginLetter%
            END IF
            FOR Letter% = BeginLetter% to EndLetter%
               DefType$(Letter%) = VarSymbol$
            NEXT Letter%
         END SELECT

'line is "", look at comment; is it a $include statement?
      ELSEIF ucase$(left$(Comment$, 10)) = "'$INCLUDE:" THEN
'get the name of the included file
         IncludeName$ = mid$(Comment$, 11)
         IncludeName$ = remove$(IncludeName$, any " '")

'read the include file to see if any constants are declared in it.  First
'close out LineInput.
         Done% = %True
         CALL LineInput$(IncludeFile$, Done%)
         Done% = %False
         CALL ReadIncludeFile (IncludeName$)
'find our place again in the main file.
         FOR Ctr% = 1 to LineCtr%
            CALL LineInput$(IncludeFile$, Done%)
         NEXT Ctr%
      END IF
   LOOP UNTIL istrue Done% and Remainder$ = ""
'not sure what's going on here.  Usually printing string$(pbvScrnCols, " ")
'with semicolon will not push the cursor to the next line.  Sometimes if I
'have changed from 25 to 50 lines per screen, it does.  Avoid the problem for
'now.
   locate, 1
   print string$(pbvScrnCols - 1, " ");
   locate, 1
   print IncludeFile$; " read"
END SUB

FUNCTION Trim$(InputString$)
'-----------------------------------------------------------------------------
'Description:  uppercases and trims leading and trailing blanks.  It
'              saves me typing "ucase(ltrim$(rtrim$(..." all the time.
'-----------------------------------------------------------------------------

   Trim$ = ucase$(rtrim$(ltrim$(InputString$)))
END FUNCTION
