'
'ͻ
' Source File    X10_SHEL.BAS                                              
'                                                                          
' System         X-10 Professional Developers Library for QuickBasic 4.5   
'                                                                          
' Purpose        Example program shell for Home Automation using X-10      
'                modules and a computer serial port interface.             
'                                                                          
' Author         Dale Morrison                                             
' Copyright      Smart Home Technology, 1993                               
'ͼ
'
'Ŀ
' Load the file containing declarations and constants for the X10_PDLB.LIB  
'
'$INCLUDE: '\QB45\X10_PDLB\X10_PDLB.BI'

'Ŀ
' Your basic clear the screen command                                       
'
CLS

'Ŀ
' Define your custom variables that are initialized in the Startup procedure
'
DIM SHARED DayOff

'Ŀ
' Define the interrupt address for the x10 driver                           
'
Addr523% = &H60

'Ŀ
' Open communications to the resident TW523 driver                          
'
OpenTW523 SinglePhase

'Ŀ
' Do your Startup procedure to initialize your modules and variables        
'
Startup

'Ŀ
' Start a timer to be used for saving the status of the modules             
'
StartTimer 0, 60

'Ŀ
' Now check to see if there was a power outage since the last boot up       
'
IF PowerOutage% THEN
    LoadStatus
    PlayCatchup
END IF

'
'Ŀ
' Start the loop to monitor your schedule until a keypress                  
'
DO

    'Ŀ
    ' Get the current system time based on the 24 hour clock, ie: 23:00     
    '
    GetX10Time

    'Ŀ
    ' Get the next received X-10 command from the driver, Null if none      
    '
    GetX10Cmd

    'Ŀ
    ' If you want to see the time and date when the system catches up       
    '
    IF PowerOutage% THEN
        LOCATE , 4
        PRINT LEFT$(X10Time$, 5) + "  " + X10Dow$ + "  " + X10Date$;
        LOCATE , 4
    END IF

    'Ŀ
    ' The part of your schedule that deals with time of day events          
    '

    IF TimeIs%("00:01") THEN
        LoadSunTimes
        GetX10Dow
        GetX10Date
        DisplayX10Globals
    END IF

    'Ŀ
    ' The part of your schedule that deals with the received X-10 commands  
    '

    IF X10CmdIs%("A01 On") THEN

       '...do something here...

    END IF

    'Ŀ
    ' The part of your schedule that would deal with any expired timers     
    '

    IF TimerExpired%(0) THEN
        SaveStatus
        StartTimer 0, 60
    END IF

    '
    'Ŀ
    ' Put any other conditions in this area.  Its wide open what you can do 
    '

    IF DateIs%("04-04") AND TimeIs%("02:00") THEN
        SetDaylightTime
    END IF

    IF DateIs%("10-25") AND TimeIs%("02:00") THEN
        SetStandardTime
    END IF
    
'Ŀ
' End of the schedule loop.  If the user pressed the x key then exit        
'
LOOP UNTIL UCASE$(INKEY$) = "X"

'Ŀ
' Make sure you close the communications to the TW523 driver before quitting
'
CloseTW523

'Ŀ
' Now you can safely quit the application                                   
'
END '

'ͻ
' Sub Program    Startup                                                   
'                                                                          
' Purpose        To initialize all of your X-10 modules to a known state.  
'                Usually this would be to turn all modules to Off.         
'                This is where you would also do any first time            
'                initialization of any variables you will be using.        
'                                                                          
' Syntax         Startup                                                   
'                                                                          
' Parameters     None                                                      
'                                                                          
' Example        Startup                                                   
'                                                                          
' Assumes        Nothing                                                   
'                                                                          
' Side Effects   Depends on what you do inside the procedure               
'                                                                          
' Returns        Nothing                                                   
'                                                                          
' LIB Calls      None                                                      
'                                                                          
'͹
' Notes:                                                                    
'                                                                           
'ͼ
'
'
'
SUB Startup
'

    'Ŀ
    ' Define the addresses of your plug-in lamp modules                     
    '
    LampModules$ = "A01A02"

    'Ŀ
    ' Initialize the variables for sunrise/sunset calcs.    ie: for Seattle 
    '
    SetMapCoord 122.2, 47.37, 8, "04-04", "10-25"

    'Ŀ
    ' Initialize whether we are currently in Daylight or Standard time      
    '
    SetDaylightFlag

    'Ŀ
    ' Load the times for the variables Dawn$, Sunrise$, Sunset$ and Dusk$   
    '
    LoadSunTimes

    'Ŀ
    ' Do you want to display what is being transmitted and received?        
    '
    ShowTransmit% = Yes

    '
    'Ŀ
    ' Turn all modules off, including appliance modules                     
    '
    TurnAllModules "D", xOff
    TurnAllModules "E", xOff
    TurnAllModules "F", xOff
    TurnAllModules "G", xOff
    TurnAllModules "O", xOff
    
    'Ŀ
    ' Do your custom variable initializations, ie: your days off            
    '
    IF DowIs%("Mon") OR DowIs%("Tue") THEN
        DayOff = True
    END IF

    DisplayX10Globals
    
END SUB
